#!/usr/bin/env python3
import argparse
import random
import shutil
import subprocess

def parser_args():
    parser = argparse.ArgumentParser('Create traning, validation, and test files from the inputs file.')
    parser.add_argument('inputs', help='a camma-seperated list of the files of one data set')
    parser.add_argument('prefix', help='output files would be prefix.tr, prefix.va, and prefix.te')
    return vars(parser.parse_args())

def partition(in_path, out_path1, out_path2, ratio1):
    random.seed(0)
    with open(in_path, 'rt') as f_in,\
            open(out_path1, 'wt') as f_out1,\
            open(out_path2, 'wt') as f_out2:
        for line in f_in:
            if random.random() < ratio1:
                f_out1.write(line)
            else:
                f_out2.write(line)

if __name__ == '__main__':
    args = parser_args()
    inputs = args['inputs'].split(',')
    if len(inputs) == 1:
        # 1. do 80/20 split to get training and test files
        # 2. split training file into training (80%) and validation (20%) files
        # 3. search parameters using training and validation
        # 4. combine training and validation to training a model, and
        #    calculate the test logarithmic loss
        print('I only see one input file. It must be the whole data set.')
        partition(inputs[0], args['prefix']+'.tr', args['prefix']+'.te', 0.8)
        partition(args['prefix']+'.tr', args['prefix']+'.va.tr', args['prefix']+'.va.te', 0.8)
    elif len(inputs) == 2:
        # 1. do 80-20 split for training file
        # 2. search parameters and then
        print('Two data files are found. We have one for training and the other one for test.')
        shutil.copy(inputs[0], args['prefix']+'.tr')
        shutil.copy(inputs[1], args['prefix']+'.te')
        partition(inputs[0], args['prefix']+'.va.tr', args['prefix']+'.va.te', 0.8)
    else:
        print('Unsupported case')
